import curses
from time import sleep
import cv2

from camshell.xscreen import XScreen


class CamShell(XScreen):
    def __init__(
        self,
        cap_id: int | str,
        mono_color: bool = False,
        screen: curses.window | None = None,
    ) -> None:
        super().__init__(mono_color, screen)
        self.cap_id: int | str = cap_id
        self.cap: cv2.VideoCapture = None

    def render(self) -> None:
        image = self.get_image()
        resized_frame = cv2.resize(
            image, (self.width, self.height), interpolation=cv2.INTER_AREA
        )
        self.process_image(resized_frame)
        self.display()

    def get_image(self) -> cv2.UMat:
        ret, frame = self.cap.read()
        if not ret:
            raise ValueError("Error: Could not read frame.")
        return frame

    def process_image(self, image: cv2.UMat) -> None:
        display_height, display_width = self.height, self.width
        image_height, image_width, _ = image.shape
        aspect_ratio = image_width / image_height

        if aspect_ratio > (display_width / display_height):
            target_width = display_width
            target_height = int(display_width / aspect_ratio)
        else:
            target_height = display_height
            target_width = int(display_height * aspect_ratio)

        assert target_width <= display_width, "Target width exceeds display width."
        assert target_height <= display_height, "Target height exceeds display height."
        resized_image = cv2.resize(
            image, (target_width, target_height), interpolation=cv2.INTER_AREA
        )
        self.submit_to_screen(resized_image)

    def run(self, screen: curses.window) -> None:
        if self.screen is None:
            self.init(screen)

        while True:
            try:
                self.render()
            # except ResizeScreenError:
            #     pass
            except KeyboardInterrupt:
                break
            sleep(0.1)

    def __enter__(self) -> "CamShell":
        cap = cv2.VideoCapture(self.cap_id)
        if not cap.isOpened():
            raise ValueError("Error: Could not open video device.")
        self.cap = cap
        return self

    def __exit__(self, exc_type, exc_value, traceback) -> None:
        self.cap.release()
        cv2.destroyAllWindows()

    @classmethod
    def start(cls, cap_id: int | str, mono_color: bool = False) -> None:
        try:
            cap_id = int(cap_id)
        except ValueError:
            pass

        with cls(cap_id=cap_id, mono_color=mono_color) as x:
            curses.wrapper(x.run)
