from typing import Generator, Iterable
import numpy as np


class PixShell:
    """A Shell Pixel"""

    DISPLAY_CHARS = " .`^',:-_\"~+=<*>x#%@&$"
    DISPLAY_CHARS_LEN = len(DISPLAY_CHARS)

    def __init__(self, mono_color: bool = False) -> None:
        self.width = None
        self.height = None
        self.__pixels = None
        self.mono_color = mono_color
        self.DISPLAY_CHARS_LEN = len(self.DISPLAY_CHARS)

    def init(self, width: int, height: int) -> None:
        self.width = width
        self.height = height

        # The pixel matrix is in this format:
        # [intensity_char_id, terminal_color_id] x width x height
        self.__pixels = np.zeros((height, width, 2), dtype=np.uint8)

    @staticmethod
    def rgb_to_intensity_id(r: int, g: int, b: int) -> int:
        """Convert RGB to intensity."""
        value = (r * 0.299 + g * 0.587 + b * 0.114) / 255
        return int(value * (PixShell.DISPLAY_CHARS_LEN - 1))

    def rgb_to_terminal_color_id(self, r: int, g: int, b: int) -> int:
        """Convert RGB to terminal color."""
        return 16 + (36 * (r // 51)) + (6 * (g // 51)) + (b // 51)

    def rgb_to_value(self, r: int, g: int, b: int) -> tuple[int, int]:
        """Convert RGB to intensities string. Format {intensity-char-id}{color-id}"""
        assert 0 <= r <= 255, "Red value must be between 0 and 255."
        assert 0 <= g <= 255, "Green value must be between 0 and 255."
        assert 0 <= b <= 255, "Blue value must be between 0 and 255."
        intensity_char_id = self.rgb_to_intensity_id(r, g, b)
        if self.mono_color:
            return intensity_char_id, 127
        return intensity_char_id, self.rgb_to_terminal_color_id(r, g, b)

    def set(self, x: int, y: int, rgb: tuple[int, int, int]) -> None:
        """Set a pixel at x, y with RGB color."""
        assert self.__pixels is not None, "PixelShell not initialized."
        assert 0 <= x < self.width, "X coordinate out of bounds."
        assert 0 <= y < self.height, "Y coordinate out of bounds."

        intensity_char_id, terminal_color_id = self.rgb_to_value(*rgb)
        self.__pixels[y, x] = [intensity_char_id, terminal_color_id]

    def get(self, x: int, y: int) -> tuple[int, int]:
        """Get pixel value at x, y."""
        assert self.__pixels is not None, "PixelShell not initialized."
        assert 0 <= x < self.width, "X coordinate out of bounds."
        assert 0 <= y < self.height, "Y coordinate out of bounds."
        return tuple(self.__pixels[y, x])

    def get_str(self, x: int, y: int) -> tuple[str, int]:
        """Get pixel value at x, y."""
        intensity_char_id, col = self.get(x, y)
        return self.DISPLAY_CHARS[intensity_char_id], col

    def get_row(self, y: int) -> Generator[tuple[str, int], None, None]:
        """Get a row of pixels."""
        for x in range(self.width):
            yield self.get_str(x, y)

    def submit_to_screen(
        self, resized_image: Iterable[Iterable[tuple[int, int, int]]]
    ) -> None:
        for y, row in enumerate(resized_image):
            for x, (b, g, r) in enumerate(row):
                self.set(x, y, (r, g, b))
